import pickle
import os
import os.path
import string
import copy
import logging
import kraken.Constants
from kraken.entrytypes.Entry import loadKraEntry
from EntryBase import EntryBase
import MetaDataDescriptor

class MetaData:

    def __init__(self):
       self.entries = [] # the loaded data
       self.keywords = []
       self.descriptors = MetaDataDescriptor.generateMetaDataDescriptors()
    
    def writeToRdf(self, fileName):
        base = EntryBase()
        store = base.createStore()
        for entry in self.entries:
            entry.writeEntryToRdf(store=store, fileName=None)
        base.saveStoreAndPostProcess(store, fileName)
     
    def load(self, system, runtime):
        print "Generating the metadata..."
        for a in system.getProperty("krakenDirs"):
            self.__loadOneDir(a, system, runtime)        
        print "Done."
    
    def addEntry(self, entry):
        self.entries.append(copy.deepcopy(entry))
        self.__updateKeywords(entry)

    '''
      Updates an older occurence of the entry with the new one.
      Used for in-place updating for annotate
    '''
    def updateEntry(self, entry):
        counter = 0
        for a in self.entries:
            if a.getResourceURI() == entry.getResourceURI():
                print "Entry found, updating..."
                self.entries[counter] = entry
                break
            counter += 1
        self.__updateKeywords(entry)
        
        
    def __loadOneDir(self, directory, system, runtime):
        logging.warning("Processing directory "+ directory)
        if not os.path.isdir(directory):
            logging.critical("Directory "+ directory + " does not exist! Skipping...")
            return
        for x in os.listdir(directory):
           entryDir = os.path.join(directory,x)
           if not os.path.isdir(entryDir):
               continue
           # try if the kraken file is there 
           entry = loadKraEntry(system, entryDir)
           # This is for converting old style entries
           if (entry == None):
              logging.warning("This doesn't look as kraken dir")
              continue
                 
           # FIXME: this is for converting older files.           
           # entry.directoryName = x                    
           # entry.writeRDF(directory + x + "/"+kraken.Constants.FILENAME_ENTRY_RDF)
           # End FIXME
           entry.krakenDirName = directory
           self.entries.append(entry)
           self.__updateKeywords(entry)
        #print "Keywords: " + str(self.keywords)
                           
    def __updateKeywords(self, entry):
        for a in entry.keywords:
           b = string.strip(a)
           if b not in self.keywords:
              self.keywords.append(b) # needed because of some old errors

    def __str__(self):
        x = "Summary of the kraken:"
        x = x + "\n\tThere are " + str(len(self.entries)) + " entries."
        x = x + "\n\tKeywords: " + str(self.keywords)
        return x                        

def generateCache(system, runtime):
    filename = os.path.expanduser(kraken.Constants.METADATA_FILE)
    temp = MetaData()
    temp.load(system, runtime)
    __writeToFile(temp, filename)
    return temp

# 
#  Used to write out the cache after it was updated with addEntry
#  updateEntry or other stuff like that 
# 
def updateCache(system, runtime, metadata):
    filename = os.path.expanduser(kraken.Constants.METADATA_FILE)
    __writeToFile(metadata, filename)
           
def loadMetaData(system, runtime):
  filename = os.path.expanduser(kraken.Constants.METADATA_FILE)
  if runtime.forceRegenerateCache:
      return generateCache(system, runtime)
  try:    
    FILE = open(filename)
    temp = pickle.load(FILE)
    FILE.close()
    # FIXME: check if what is in the file, is metadata
    return temp
  except:
    print "Can not open metadata file " + filename
    return generateCache(system, runtime)
    
def __writeToFile(metadata, filename):
   filename = os.path.expanduser(filename)
   FILE = open(filename, "w")
   pickle.dump(metadata, FILE)
   FILE.close()          
    # and now writing to the meta RDF file    
   metadata.writeToRdf(os.path.expanduser(kraken.Constants.METADATA_RDF_FILE))
    
