from kraken.entrytypes.MetaData import MetaData
import copy

class Filter:
    def filterMetaData(self, metadata):
       temp = MetaData()
       temp.keywords = copy.deepcopy(metadata.keywords)
       for a in metadata.entries:
          if self.matches(a):
             # print "Appending" + str(a)
             temp.entries.append(copy.deepcopy(a))
       return temp
    
class NoFilter(Filter):
    def filterMetaData(self, metadata):
        return copy.deepcopy(metadata)
    
class MultiFilter(Filter):
    def __init__(self):
        self.filters = []

        
class AndFilter(MultiFilter):    
    def __init__(self):
        MultiFilter.__init__(self)
        
    def matches(self, entry):
        for flt in self.filters:
            if not flt.matches(entry):
                return 0
        return 1

        
class OrFilter(MultiFilter):
    def __init__(self):
        MultiFilter.__init__(self)
        
    def matches(self, entry):
        for flt in self.filters:
            if flt.matches(entry):
                return 1
        return 0

class NotFilter(Filter):
    def __init__(self):
        flt = None
        
    def matches(self, entry):
        return not flt.matches(entry)
        
        
'''
    This filter extracts a value from the entry.
'''        
class EvaluationFilter(Filter):
    def __init__(self):
        self.propertyName = None
        self.expression = None
        # the default fallback is false
        self.fallback = 0
        self.filterValue = None

    def getValue(self, entry):
        if self.propertyName != None:
            return entry.getProperty(self.propertyName)
        #if self.expression != None:
        #    return eval expression
        raise "Unable to evaluate!"
        
class EqualityFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) == self.filterValue:
            return 1
        return 0
        
class SmallerFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) < self.filterValue:
            return 1
        return 0
        
class SmallerOrEqualFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) <= self.filterValue:
            return 1
        return 0
        
class GreaterFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) > self.filterValue:
            return 1
        return 0

class GreaterOrEqualFilter(EvaluationFilter):
    def matches(self, entry):
        if self.getValue(entry) <= self.filterValue:
            return 1
        return 0
        
'''
   Matches if any of the terms in the filterValue is in the sequence
'''        
class ContainsAnyFilter(EvaluationFilter):
    def matches(self, entry):
        val = self.getValue(entry)
        for a in self.filterValue:
            if a in val:
                return 1
        return 0
            
'''
   Matches if all of the terms in the filterValue is in the sequence
'''        
class ContainsAllFilter(EvaluationFilter):
    def matches(self, entry):
        val = self.getValue(entry)
        #print val
        for a in self.filterValue:
            if not (a in val):
                return 0
        return 1
        
'''
   Matches if the filterValue and the specified sentence are the same 
   except for order
'''        
class SameSetFilter(EvaluationFilter):    
    def matches(self, entry):
        val = self.getValue(entry)
        if len(val) != len(self.filterValue):
            return 0
        for a in self.filterValue:
            if not (a in val):
                return 0
        return 1
        