from kraken.entrytypes.MetaData import MetaData
import string
import copy

def compare(a, b, ranker):
    # print ranker
    return ranker.compare(a, b)

def rankMetaData(metadata, ranker):
    md = copy.deepcopy(metadata)
    if ranker.__class__.__name__ != "NoRanking":    
       md.entries.sort(lambda a, b: compare(a, b, ranker))
    else:
       print "No ranking needed!"
    if ranker.limit != -1:
       md.entries = md.entries[0..ranker.limit]
    return md
    
class Ranker:
    def __init__(self):
        self.limit = -1
       
    def compare(self, a, b):
        return True

    def __str__(self):
        if self.limit != -1: 
           return "Limit = " + self.limit
        return ""

class NoRanking(Ranker):
    pass
        
        
class ReverseRanker(Ranker):
    def __init__(self):
        self.theSorter = None
        
    def compare(self, a, b):
        return theSorter.compare(b,a)
        
    def __str__(self):
        return "Reverse order" + string.replace("\n"+str(theSorter),"\n", "\n\t") 
        
class EvaluationRanker(Ranker):
    def __init__(self):
        self.propertyName = None
        
    def compare(self, a, b):
        aValue = a.getProperty(propertyName)
        bValue = b.getProperty(propertyName)
        return cmp(a, b)

    def __str__(self, a, b):
        return "Rank by property: " + propertyName        

        
'''
   Take the sorters one by one. If one is equal, pick the next one...
'''        
class FallbackRanker(Ranker):
    def __init__(self):
        self.rankers = []
        
    def compare(self, a, b):
        for srtr in rankers:
            val = srtr.compare(a,b)
            if val != 0:
                return val
            # else, go to the next one.     
        return 1
        
'''
   Preferred value sorter - some preferred values for enumeration type variables...
   Or for sequences
'''        
class PreferredValueRanker(Ranker):
    def __init__(self):
        self.preferredValues = []
        self.notPreferredValues = []
        
    def compare(self, a, b):
        if a == b:
            return 0
        for val in preferredValues:
            if val == a:
                return 1
            if val == b:
                return -1
        for val in notPreferredValues:
            if val == a:
                return -1
            if val == b:
                return 1
        
''' 
   Linear combination sorter
   FIXME: I don't know how to do this, but...
'''